﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[System.Serializable]
public class BlaObject
{
    public GameObject go;
    public bool shouldExistInProgress;
}

[System.Serializable]
public class ObjectGroup
{
    public List<BlaObject> gameObjects;
}

public class Blocker : FocusHolder, ISwichClient
{
    public ObjectGroup solidObjectGroup;
    public ObjectGroup unsolidObjectGroup;
    public ObjectGroup solidToUnsolidGroup;
    public ObjectGroup unsolidToSolidGroup;

    public bool antiMatter;
    public float holdForToggle;

    public float rotationSpeed;
    public float rotationSegment;

    public AudioSource lookedAtAudioSource;
    public AudioSource nonLookedAtAudioSource;
    public AudioSource rotationAudioSource;

    private bool solidState;
    private bool canChange = true;

    private bool HasSwitch;
    private float targetAngle;

    protected void Start()
    {
        targetAngle = transform.rotation.eulerAngles.z;
        ChanceState(antiMatter, false);
    }

    protected void Update()
    {
        base.Update();

        bool shouldFeedback = true;
        if (TimeHadFocus >= holdForToggle)
        {
            if (solidState != !antiMatter)
            {
                ChanceState(!antiMatter, true);
            }

            shouldFeedback = false;
            TimeHadFocus = holdForToggle;
        }
        else if(TimeHadFocus <= 0)
        {
            if (solidState != antiMatter)
            {
                ChanceState(antiMatter, true);
            }

            shouldFeedback = false;
        }

        if (!shouldFeedback)
        {
            foreach (var bla in solidToUnsolidGroup.gameObjects)
            {
                bla.go.SetActive(false);
            }
            foreach (var bla in unsolidToSolidGroup.gameObjects)
            {
                bla.go.SetActive(false);
            }
        }
        else
        {
            foreach (var bla in solidToUnsolidGroup.gameObjects)
            {
                var shouldBeActive = solidState;
                bla.go.SetActive(shouldBeActive);

                if (shouldBeActive)
                {
                    SetProgressRenderer(bla);
                }
            }

            foreach (var bla in unsolidToSolidGroup.gameObjects)
            {
                var shouldBeActive = !solidState;
                bla.go.SetActive(shouldBeActive);

                if (shouldBeActive)
                {
                    SetProgressRenderer(bla);
                }
            }

            foreach (var bla in solidObjectGroup.gameObjects)
            {
                if (bla.shouldExistInProgress)
                {
                    bla.go.SetActive(true);

                    SetProgressRenderer(bla);
                }
            }
        }

        var rotAngles = transform.rotation.eulerAngles;
        if (HasSwitch && Mathf.DeltaAngle(rotAngles.z, targetAngle) < 0.001f)
        {
            targetAngle = (targetAngle + rotationSegment) % 360.0f;
            rotationAudioSource.Play();
        }

        rotAngles.z = Mathf.LerpAngle(rotAngles.z, targetAngle, rotationSpeed);
        transform.rotation = Quaternion.Euler(rotAngles);
    }

    private void SetProgressRenderer(BlaObject bla)
    {
        var ren = bla.go.GetComponent<SpriteRenderer>();
        if (ren)
        {
            var c = ren.color;
            var f = TimeHadFocus/holdForToggle;
            c.a = antiMatter ? 1 - f : f;
            ren.color = c;
        }
    }

    void ChanceState(bool solid, bool playAudio)
    {
        foreach (var obj in unsolidObjectGroup.gameObjects)
        {
            obj.go.SetActive(!solid);
        }
        
        foreach (var obj in solidObjectGroup.gameObjects)
        {
            obj.go.SetActive(solid);
        }

        if (playAudio)
        {
            AudioSource audioS = solid ? lookedAtAudioSource : nonLookedAtAudioSource;
            audioS.Play();
        }

        solidState = solid;
        canChange = false;

    }

    public void OnSwitchStateChanged(bool newState)
    {
        HasSwitch = newState;
    }
}