﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

interface IEyeFocusable
{
    void OnFocusChanged(bool hasFocus);
}

public class EyeControl : MonoBehaviour
{
    public Vector2 eyeTrackingWarningTexture;
    public Texture2D eyeTrackerGazeLost;
    public Texture2D eyeTrackerNotConnected;

    private GUIStyle m_BackgroundStyle = new GUIStyle();        // Style for background tiling
    
    public bool useMouseControl;
    public float lerpSpeed;
    public float castRadius;

    private Collider2D[] previousColliders;

    void Awake()
    {
        Tobii.EyeTracking.EyeTracking.Initialize();
        previousColliders = new Collider2D[0];
    }
    
	void Update ()
    {
        var gazePoint = Tobii.EyeTracking.EyeTracking.GetGazePoint();
        var pos2D = (Vector2)Input.mousePosition;
        if (gazePoint != null && gazePoint.IsValid && !useMouseControl && gazePoint.IsWithinScreenBounds)
	    {
            pos2D = gazePoint.Screen;
        }

        var position = Camera.main.ScreenToWorldPoint(pos2D);
        position.z = 0;

        var newColliders = UnityEngine.Physics2D.OverlapCircleAll(position, castRadius);
        foreach (var cl in previousColliders)
        {
            if (cl)
            {
                var focusable = cl.GetComponent<IEyeFocusable>();
                if (focusable != null && !newColliders.Contains(cl))
                {
                    focusable.OnFocusChanged(false);
                }
            }
        }

        foreach (var cl in newColliders)
        {
            var focusable = cl.GetComponent<IEyeFocusable>();
            if (focusable != null && !previousColliders.Contains(cl))
            {
                focusable.OnFocusChanged(true);
            }
        }

        previousColliders = newColliders;

        transform.position = Vector3.Lerp(transform.position, position, lerpSpeed);
    }

    void OnGUI()
    {
        Tobii.EyeTracking.GazeTracking gazeTracking = Tobii.EyeTracking.EyeTracking.GetGazeTrackingStatus();
        if (!gazeTracking.IsTrackingEyeGaze)
        {
            GUI.depth = -1001;

            const float border = 10f;
            var original = GUI.color;
            var c = original;
            c.a = 0.5f + (1 + Mathf.Sin(Time.time * 2)) / 4;
            GUI.color = c;
            var rect = new Rect(Screen.width - eyeTrackingWarningTexture.x - border*2, border, eyeTrackingWarningTexture.x, eyeTrackingWarningTexture.y);
            GUI.DrawTexture(rect, eyeTrackerGazeLost, ScaleMode.ScaleToFit);
            GUI.color = original;
        }
    }
}
