﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Spawner : FocusHolder
{
    public AudioSource spawnAudioSource;
    public GameObject projPrefab;
    public Transform spawnTransform;
    public float spawnProjectileTimeThreshold;
    public float rotateAnglesPerSecond;

    public ParticleSystem onObject;
    public ParticleSystem offObject;

    private float spawnIn;

    // Update is called once per frame
    protected override void Update ()
    {
        base.Update();

        transform.Rotate(Vector3.forward, rotateAnglesPerSecond * Time.deltaTime);

        if (spawnIn <= 0 && HasFocus)
        {
            spawnIn = spawnProjectileTimeThreshold;
        }

        if (spawnIn > 0)
        {
            spawnIn -= Time.deltaTime;

            if (spawnIn <= 0)
            {
                SpawnProjectile();
            }
        }

        var focus = spawnIn > 0;
        onObject.gameObject.SetActive(focus);
        offObject.gameObject.SetActive(!focus);

    }

    protected void SpawnProjectile()
    {
        var angle = transform.rotation.eulerAngles.z * Mathf.Deg2Rad;
        var proj = Instantiate(projPrefab, spawnTransform.position, Quaternion.identity).GetComponent<Projectile>();

        var dir = new Vector2(Mathf.Cos(angle), Mathf.Sin(angle));
        proj.SetVelocity(dir);

        onObject.Clear();
        onObject.Play();

        spawnAudioSource.Play();

        ResetFocusTimer();
    }
}
