public enum ButtonType
{
    StartGame,
    QuitGame,
    Credits,
    MainMenu
}

public class UIButton : FocusHolder
{
    public LevelProgression levelProgression;
    public ButtonType buttonType;
    public float doActionAfter;

    public ObjectGroup solidObjectGroup;
    public ObjectGroup unsolidObjectGroup;
    public ObjectGroup solidToUnsolidGroup;
    public ObjectGroup unsolidToSolidGroup;

    public bool antiMatter;
    public float holdForToggle;

    public UnityEngine.AudioSource lookedAtAudioSource;
    public UnityEngine.AudioSource nonLookedAtAudioSource;

    private bool solidState;

    protected void Start()
    {
        ChanceState(antiMatter, false);
    }

    protected void Update()
    {
        base.Update();

        bool shouldFeedback = true;
        if (TimeHadFocus >= holdForToggle)
        {
            if (solidState != !antiMatter)
            {
                ChanceState(!antiMatter, true);
            }

            shouldFeedback = false;
            TimeHadFocus = holdForToggle;
        }
        else if (TimeHadFocus <= 0)
        {
            if (solidState != antiMatter)
            {
                ChanceState(antiMatter, true);
            }

            shouldFeedback = false;
        }

        if (!shouldFeedback)
        {
            foreach (var bla in solidToUnsolidGroup.gameObjects)
            {
                bla.go.SetActive(false);
            }
            foreach (var bla in unsolidToSolidGroup.gameObjects)
            {
                bla.go.SetActive(false);
            }
        }
        else
        {
            foreach (var bla in solidToUnsolidGroup.gameObjects)
            {
                var shouldBeActive = solidState;
                bla.go.SetActive(shouldBeActive);

                if (shouldBeActive)
                {
                    SetProgressRenderer(bla);
                }
            }

            foreach (var bla in unsolidToSolidGroup.gameObjects)
            {
                var shouldBeActive = !solidState;
                bla.go.SetActive(shouldBeActive);

                if (shouldBeActive)
                {
                    SetProgressRenderer(bla);
                }
            }

            foreach (var bla in solidObjectGroup.gameObjects)
            {
                if (bla.shouldExistInProgress)
                {
                    bla.go.SetActive(true);

                    SetProgressRenderer(bla);
                }
            }
        }
    }

    private void SetProgressRenderer(BlaObject bla)
    {
        var ren = bla.go.GetComponent<UnityEngine.SpriteRenderer>();
        if (ren)
        {
            var c = ren.color;
            var f = TimeHadFocus / holdForToggle;
            c.a = antiMatter ? 1 - f : f;
            ren.color = c;
        }
    }

    void ChanceState(bool solid, bool playAudio)
    {
        foreach (var obj in unsolidObjectGroup.gameObjects)
        {
            obj.go.SetActive(!solid);
        }

        foreach (var obj in solidObjectGroup.gameObjects)
        {
            obj.go.SetActive(solid);
        }

        if (playAudio)
        {
            UnityEngine.AudioSource audioS = solid ? lookedAtAudioSource : nonLookedAtAudioSource;
            audioS.Play();
        }

        solidState = solid;


        if (solid)
        {
            CameraFade.StartFade(1.0f, doActionAfter);

            StartCoroutine(Exit.ExecuteAfterTime(DoButtonAction, doActionAfter));
        }
    }

    private void DoButtonAction()
    {
        switch (buttonType)
        {
            case ButtonType.StartGame:
                OnStartGame();
                break;
            case ButtonType.QuitGame:
                OnQuitGame();
                break;
            case ButtonType.Credits:
                LoadCredits();
                break;
            case ButtonType.MainMenu:
                LoadMainMenu();
                break;
            default:
                throw new System.ArgumentOutOfRangeException();
        }
    }

    public void OnStartGame()
    {
        UnityEngine.SceneManagement.SceneManager.LoadScene(levelProgression.scenes[0].sceneName);
    }

    // When quit button is pressed
    public void OnQuitGame()
    {
        UnityEngine.Application.Quit();
    }

    // When credits button is pressed
    public void LoadCredits()
    {
        UnityEngine.SceneManagement.SceneManager.LoadScene(levelProgression.creditsScene);
    }

    // When credits button is pressed
    public void LoadMainMenu()
    {
        UnityEngine.SceneManagement.SceneManager.LoadScene(levelProgression.mainMenuScene);
    }
}